//--------------------------------------------------------------------------------------
//  Plik I2O1W3.cpp
//  Autor: Piotr M. Szczypinski
//  Instytut Elektroniki Politechniki Ldzkiej
//--------------------------------------------------------------------------------------

#include <streams.h>    
#include <initguid.h>
#include "I2O1W3.h"    

//--------------------------------------------------------------------------------------
//!!! Numer CLSID/GUID filtru wygenerowany przez program GuidGen
// {C5881295-2485-49c2-BDAA-166C0139A4C9}
DEFINE_GUID(CLSID_I2O1W3, 
0xc5881295, 0x2485, 0x49c2, 0xbd, 0xaa, 0x16, 0x6c, 0x1, 0x39, 0xa4, 0xc9);


//--------------------------------------------------------------------------------------
// Dane potrzebne do utworzenia filtru i jego wyprowadzen
const AMOVIESETUP_MEDIATYPE
sudPinTypes =   { &MEDIATYPE_NULL                // clsMajorType
, &MEDIASUBTYPE_NULL }  ;       // clsMinorType





const AMOVIESETUP_PIN psudPins[] =
{
	{
		L"InLeft", 
			FALSE,       
			FALSE,          
			FALSE,        
			FALSE,        
			&CLSID_NULL,       
			L"Output",       
			1,               
			&sudPinTypes },   
		{
			L"InRight",   
				FALSE,           
				FALSE,           
				FALSE,          
				FALSE,        
				&CLSID_NULL,    
				L"Output",     
				1,           
				&sudPinTypes },  
			{ L"Output",      
			FALSE,         
			TRUE,            
			FALSE,          
			FALSE,           
			&CLSID_NULL,   
			L"Input",           
			1,                 
			&sudPinTypes    
			}
};


const AMOVIESETUP_FILTER
sudNullIP = { &CLSID_I2O1W3                  // clsID
, L"Wielordzenie_I2O1W3"                    // strName
, MERIT_DO_NOT_USE                // dwMerit
, 3                               // nPins
, psudPins };                     // lpPin

//--------------------------------------------------------------------------------------
// Potrzebne podczas wykonania CreateInstance
CFactoryTemplate g_Templates[]=
{   {L"Wielordzenie_I2O1W3"
, &CLSID_I2O1W3
, KlasaFiltru_I2O1W3::CreateInstance
, NULL
, &sudNullIP }
};
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);

//--------------------------------------------------------------------------------------
// Konstuktor klasy filtru
KlasaFiltru_I2O1W3::KlasaFiltru_I2O1W3(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
: CTwoOneFiltr (tszName, punk, CLSID_I2O1W3)
{
	probka_WejsciowaWstrzymana[0] = NULL; //!!!
	probka_WejsciowaWstrzymana[1] = NULL; //!!!
	event_WejsciowaGotowa[0] = CreateEvent(NULL, FALSE, FALSE, TEXT("ProbkaLewaCzeka"));
	event_WejsciowaGotowa[1] = CreateEvent(NULL, FALSE, FALSE, TEXT("ProbkaPrawaCzeka"));
	event_ZwolnijProbke[0] = CreateEvent(NULL, FALSE, FALSE, TEXT("ZwolnijLewa"));
	event_ZwolnijProbke[1] = CreateEvent(NULL, FALSE, FALSE, TEXT("ZwolnijPrawa"));
	Create(); //!!! uruchamianie watku
} 

//--------------------------------------------------------------------------------------
// Destruktor klasy filtru
KlasaFiltru_I2O1W3::~KlasaFiltru_I2O1W3()
{
	CallWorker(1); //!!! zamykanie watku
	CloseHandle(event_ZwolnijProbke[1]);
	CloseHandle(event_ZwolnijProbke[0]);
	CloseHandle(event_WejsciowaGotowa[1]);
	CloseHandle(event_WejsciowaGotowa[0]);
}

//--------------------------------------------------------------------------------------
// Tworzy obiekt COM filtru Zielenina
CUnknown * WINAPI KlasaFiltru_I2O1W3::CreateInstance(LPUNKNOWN punk, HRESULT *phr) 
{
	ASSERT(phr);

	KlasaFiltru_I2O1W3 *pNewObject = new KlasaFiltru_I2O1W3(NAME("Wielordzenie_I2O1W3"), punk, phr );
	if(pNewObject == NULL) 
		if(phr)
			*phr = E_OUTOFMEMORY;

	return pNewObject;
}

HRESULT KlasaFiltru_I2O1W3::DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProperties)
{
   	CheckPointer(pAlloc, E_POINTER);
	CheckPointer(pProperties, E_POINTER);

	HRESULT hr = NOERROR;

	VIDEOINFO *pvi = (VIDEOINFO *) m_pOutput->CurrentMediaType().Format();
	pProperties->cBuffers = 1;
	pProperties->cbBuffer = pvi->bmiHeader.biSizeImage;

	ASSERT(pProperties->cbBuffer);

	ALLOCATOR_PROPERTIES Actual;
	hr = pAlloc->SetProperties(pProperties,&Actual);
	if(FAILED(hr)) return hr;
	if(Actual.cbBuffer < pProperties->cbBuffer) return E_FAIL;
	ASSERT(Actual.cBuffers == 1);
	return NOERROR;
}

//------------------------------------------------------------------------------
//!!! Metoda informujaca o dostarczanych na wyjsciu rodzajach probek
HRESULT KlasaFiltru_I2O1W3::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	if(iPosition < 0) return E_INVALIDARG;
	if(iPosition > 0) return VFW_S_NO_MORE_ITEMS;

	CheckPointer(pMediaType,E_POINTER);

	VIDEOINFO *pvi = (VIDEOINFO *) pMediaType->AllocFormatBuffer(sizeof(VIDEOINFO));
	if(NULL == pvi) return(E_OUTOFMEMORY);

	ZeroMemory(pvi, sizeof(VIDEOINFO));
	pvi->bmiHeader.biCompression = BI_RGB;
	pvi->bmiHeader.biBitCount = 24;
	pvi->bmiHeader.biSize       = sizeof(BITMAPINFOHEADER);
	pvi->bmiHeader.biWidth      = m_Width;
	pvi->bmiHeader.biHeight     = m_Height;
	pvi->bmiHeader.biPlanes     = 1;
	pvi->bmiHeader.biSizeImage  = GetBitmapSize(&pvi->bmiHeader);
	pvi->bmiHeader.biClrImportant = 0;

	SetRectEmpty(&(pvi->rcSource));
	SetRectEmpty(&(pvi->rcTarget)); 
	pMediaType->SetType(&MEDIATYPE_Video);
	pMediaType->SetFormatType(&FORMAT_VideoInfo);
	pMediaType->SetTemporalCompression(FALSE);
	const GUID SubTypeGUID = GetBitmapSubtype(&pvi->bmiHeader);
	pMediaType->SetSubtype(&SubTypeGUID);
	pMediaType->SetSampleSize(pvi->bmiHeader.biSizeImage);
	return NOERROR;
}

//--------------------------------------------------------------------------------------
//!!! Funkcja wywolywana podczas sprawdzania zgodnosci formatow wejsciowego i wyjsciowego
HRESULT KlasaFiltru_I2O1W3::CheckTransform(const CMediaType *mtIn,const CMediaType *mtOut)
{
	CheckPointer(mtIn,E_POINTER);
	CheckPointer(mtOut,E_POINTER);
	HRESULT hr;
	if(FAILED(hr = CheckInputType(mtIn)))
	{
		return hr;
	}
	if(*mtOut->FormatType() != FORMAT_VideoInfo)
	{
		return E_INVALIDARG;
	}
	if(mtIn->FormatLength() < sizeof(VIDEOINFOHEADER) ||
		mtOut->FormatLength() < sizeof(VIDEOINFOHEADER))
		return E_INVALIDARG;
	VIDEOINFO *pInput  = (VIDEOINFO *) mtIn->Format();
	VIDEOINFO *pOutput = (VIDEOINFO *) mtOut->Format();
	if(pOutput->bmiHeader.biCompression != BI_RGB) return E_INVALIDARG;
	if(pOutput->bmiHeader.biBitCount != 24) return E_INVALIDARG;
	if(pInput->bmiHeader.biCompression != BI_RGB) return E_INVALIDARG;
	if(pInput->bmiHeader.biBitCount != 24) return E_INVALIDARG;
	if(pOutput->bmiHeader.biWidth != pInput->bmiHeader.biWidth) return E_INVALIDARG; 
	if(pOutput->bmiHeader.biHeight != pInput->bmiHeader.biHeight) return E_INVALIDARG;
	return NOERROR;
}

//--------------------------------------------------------------------------------------
// Funkcja wywolywana podczas negocjacji rodzaju transmisji
HRESULT KlasaFiltru_I2O1W3::CheckInputType(const CMediaType* mtIn) 
{
	if(*mtIn->FormatType() != FORMAT_VideoInfo) return VFW_E_TYPE_NOT_ACCEPTED;
	if(!(IsEqualGUID(*mtIn->Type(), MEDIATYPE_Video))) return VFW_E_TYPE_NOT_ACCEPTED;

	if(IsEqualGUID(*mtIn->Subtype(), MEDIASUBTYPE_RGB24))
	{
		VIDEOINFO *pvi = (VIDEOINFO *) mtIn->Format();
		if (pvi->bmiHeader.biBitCount != 24) 
		{	
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
		m_Height = pvi->bmiHeader.biHeight;                 
		m_Width = pvi->bmiHeader.biWidth;
		return S_OK;
	} 
	return VFW_E_TYPE_NOT_ACCEPTED;
}

//--------------------------------------------------------------------------------------
// Funkcja wywolywana do transformowania probki (obrazu) 
//!!! Zmiana, dwa argumenty, probka wejsciowa i wyjsciowa
HRESULT KlasaFiltru_I2O1W3::Transform(IMediaSample *pInLeftSample, IMediaSample *pInRightSample, IMediaSample *pOutSample)
{
	BYTE *pInLeftData;                
	BYTE *pInRightData;                
	BYTE *pOutData;                
	long lDataLen;              
	int x, y;  
	RGBTRIPLE *prgbInLeft;            
	RGBTRIPLE *prgbInRight;            
	RGBTRIPLE *prgbOut;            

	AM_MEDIA_TYPE* pType = &m_pOutput->CurrentMediaType();
	VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pType->pbFormat;
	ASSERT(pvi);

	CheckPointer(pInLeftSample,E_POINTER);
	pInLeftSample->GetPointer(&pInLeftData);
	CheckPointer(pInRightSample,E_POINTER);
	pInRightSample->GetPointer(&pInRightData);

	CheckPointer(pOutSample,E_POINTER);
	pOutSample->GetPointer(&pOutData);

	lDataLen = pOutSample->GetSize();

	int width     = pvi->bmiHeader.biWidth;
	int height = abs(pvi->bmiHeader.biHeight);
	int bits     = pvi->bmiHeader.biBitCount;
	int stride     = (width * sizeof( RGBTRIPLE ) + 3) & -4;
	int pixels  = width * height;

	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++) 
		{
			prgbInLeft = (RGBTRIPLE*) (pInLeftData + stride * y + sizeof(RGBTRIPLE)*x);
			prgbInRight = (RGBTRIPLE*) (pInRightData + stride * y + sizeof(RGBTRIPLE)*x);
			prgbOut = (RGBTRIPLE*) (pOutData + stride * y + sizeof(RGBTRIPLE)*x);
			prgbOut->rgbtGreen = ((int)prgbInLeft->rgbtGreen + prgbInRight->rgbtGreen)/2;
			prgbOut->rgbtBlue = ((int)prgbInLeft->rgbtBlue + prgbInRight->rgbtBlue)/2;
			prgbOut->rgbtRed = ((int)prgbInLeft->rgbtRed + prgbInRight->rgbtRed)/2;
		}
	}
	return NOERROR;
}

//--------------------------------------------------------------------------------------
//!!! Funkcja funkcja otrzymuje probke (obraz) z wejscia
HRESULT KlasaFiltru_I2O1W3::ReceiveLeft(IMediaSample *pIn)
{
	if(pIn == NULL)	return E_POINTER;
	CheckPointer(pIn, E_POINTER);
	probka_WejsciowaWstrzymana[0] = pIn;
	SetEvent(event_WejsciowaGotowa[0]);
	DbgLog((LOG_TRACE,5,TEXT("Left - Czeka")));
	WaitForSingleObject(event_ZwolnijProbke[0], 200);
	DbgLog((LOG_TRACE,5,TEXT("Left - Poszlo")));
	return NOERROR;
}

//--------------------------------------------------------------------------------------
//!!! Funkcja funkcja otrzymuje probke (obraz) z wejscia
HRESULT KlasaFiltru_I2O1W3::ReceiveRight(IMediaSample *pIn)
{
	if(pIn == NULL)	return E_POINTER;
	CheckPointer(pIn, E_POINTER);
	probka_WejsciowaWstrzymana[1] = pIn;
	SetEvent(event_WejsciowaGotowa[1]);
	DbgLog((LOG_TRACE,5,TEXT("Right - Czeka")));
	WaitForSingleObject(event_ZwolnijProbke[1], 200);
	DbgLog((LOG_TRACE,5,TEXT("Right - Poszlo")));
	return NOERROR;
}

//------------------------------------------------------------------------------
//!!! Funkcja watku roboczego
DWORD KlasaFiltru_I2O1W3::ThreadProc(void)
{
	DWORD req = 0;
	while (true)
    {
		if(CheckRequest(&req)) Reply(NOERROR);
		if(req > 0)	return 0;

		DbgLog((LOG_TRACE,5,TEXT("ThreadProc - Czeka")));
		if(WaitForMultipleObjects(2, event_WejsciowaGotowa, true, 100) != WAIT_TIMEOUT)
		{
			DbgLog((LOG_TRACE,5,TEXT("ThreadProc - Poszlo")));

			IMediaSample * probka_DoWyslania;
			HRESULT hr = InitializeOutputSample(&probka_DoWyslania);
			if (FAILED(hr))
			{
				SetEvent(event_ZwolnijProbke[0]);
				SetEvent(event_ZwolnijProbke[1]);
			}
			else if (S_FALSE == hr) 
			{
				probka_DoWyslania->Release();
				m_bSampleSkipped = TRUE;
				if (!m_bQualityChanged) 
				{
	                NotifyEvent(EC_QUALITY_CHANGE,0,0);
		            m_bQualityChanged = TRUE;
			    }
				SetEvent(event_ZwolnijProbke[0]);
				SetEvent(event_ZwolnijProbke[1]);
	        }
			else
			{
				Transform(probka_WejsciowaWstrzymana[0], probka_WejsciowaWstrzymana[1], probka_DoWyslania); 
				SetEvent(event_ZwolnijProbke[0]);
				SetEvent(event_ZwolnijProbke[1]);
				m_pOutput->Deliver(probka_DoWyslania);
				probka_DoWyslania->Release();
			}
		}
	}
	SetEvent(event_ZwolnijProbke[0]);
	SetEvent(event_ZwolnijProbke[1]);
    return 1;
};

//--------------------------------------------------------------------------------------
// Funkcje wywolywane podczas rejestracji filtru przez program RegSvr32
STDAPI DllRegisterServer()
{
	return AMovieDllRegisterServer2( TRUE );
}

STDAPI DllUnregisterServer()
{
	return AMovieDllRegisterServer2( FALSE );
}


//--------------------------------------------------------------------------------------
// Funkcje niezbedne w bibliotece DLL
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

BOOL APIENTRY DllMain(HANDLE hModule, 
					  DWORD  dwReason, 
					  LPVOID lpReserved)
{
	return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}


