//------------------------------------------------------------------------------
// Nazwa pliku: vtwoinp.h
// Autor: Piotr M. Szczypinski, e-mail: pms@p.lodz.pl
// Data modyfikacji: 2012.03.15
// Copyright (c) 2012 Piotr M. Szczypinski
//------------------------------------------------------------------------------

#ifndef _VTWOINP_H_
#define _VTWOINP_H_

class CTwoOneFiltr;

// ==================================================
class CTwoOneInputPin : public CBaseInputPin
{
    friend class CTwoOneFiltr;
protected:
    CTwoOneFiltr *m_pTransformFilter;
public:
    CTwoOneInputPin(
        __in_opt LPCTSTR pObjectName,
        __inout CTwoOneFiltr *pTransformFilter,
        __inout HRESULT * phr,
        __in_opt LPCWSTR pName);
	STDMETHODIMP QueryId(__deref_out LPWSTR * Id)
    {
		return AMGetWideString(Name(), Id);
    }
    HRESULT CheckConnect(IPin *pPin);
    HRESULT BreakConnect();
    HRESULT CompleteConnect(IPin *pReceivePin);
    HRESULT CheckMediaType(const CMediaType* mtIn);
    HRESULT SetMediaType(const CMediaType* mt);
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);
    STDMETHODIMP BeginFlush(void);
    STDMETHODIMP EndFlush(void);

    STDMETHODIMP NewSegment(
                        REFERENCE_TIME tStart,
                        REFERENCE_TIME tStop,
                        double dRate);
    virtual HRESULT CheckStreaming();
public:
    CMediaType& CurrentMediaType() { return m_mt; };

};

// ==================================================
class CTwoOneOutputPin : public CBaseOutputPin
{
    friend class CTwoOneFiltr;

protected:
    CTwoOneFiltr *m_pTransformFilter;

public:
    IUnknown * m_pPosition;
    CTwoOneOutputPin(
        __in_opt LPCTSTR pObjectName,
        __inout CTwoOneFiltr *pTransformFilter,
        __inout HRESULT * phr,
        __in_opt LPCWSTR pName);

    ~CTwoOneOutputPin();
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, __deref_out void **ppv);
    STDMETHODIMP QueryId(__deref_out LPWSTR * Id)
    {
        return AMGetWideString(Name(), Id);
    }
    HRESULT CheckConnect(IPin *pPin);
    HRESULT BreakConnect();
    HRESULT CompleteConnect(IPin *pReceivePin);
    HRESULT CheckMediaType(const CMediaType* mtOut);
    HRESULT SetMediaType(const CMediaType *pmt);
    HRESULT DecideBufferSize(
                IMemAllocator * pAlloc,
                __inout ALLOCATOR_PROPERTIES *pProp);
    HRESULT GetMediaType(int iPosition, __inout CMediaType *pMediaType);
    STDMETHODIMP Notify(IBaseFilter * pSender, Quality q);
public:
    CMediaType& CurrentMediaType() { return m_mt; };
};

// ==================================================
class AM_NOVTABLE CTwoOneFiltr : 
	public CBaseFilter
{
public:
    int GetPinCount();
    CBasePin * GetPin(int n);
    STDMETHODIMP FindPin(LPCWSTR Id, __deref_out IPin **ppPin);
    STDMETHODIMP Stop();
    STDMETHODIMP Pause();

    CTwoOneFiltr(__in_opt LPCTSTR , __inout_opt LPUNKNOWN, REFCLSID clsid);
    ~CTwoOneFiltr();

	virtual HRESULT Transform(IMediaSample *pInLeft, IMediaSample *pInRight, IMediaSample *pOut) PURE;
	virtual HRESULT ReceiveLeft(IMediaSample *pIn) PURE;
	virtual HRESULT ReceiveRight(IMediaSample *pIn) PURE;

    virtual HRESULT CheckInputType(const CMediaType* mtIn) PURE;
    virtual HRESULT CheckTransform(const CMediaType* mtIn, const CMediaType* mtOut) PURE;
    virtual HRESULT DecideBufferSize(
                        IMemAllocator * pAllocator,
                        __inout ALLOCATOR_PROPERTIES *pprop) PURE;
    virtual HRESULT GetMediaType(int iPosition, __inout CMediaType *pMediaType) PURE;
    virtual HRESULT StartStreaming();
    virtual HRESULT StopStreaming();
    virtual HRESULT AlterQuality(Quality q);
    virtual HRESULT SetMediaType(PIN_DIRECTION direction,const CMediaType *pmt);
    virtual HRESULT CheckConnect(PIN_DIRECTION dir,IPin *pPin);
    virtual HRESULT BreakConnect(PIN_DIRECTION dir);
    virtual HRESULT CompleteConnect(PIN_DIRECTION direction,IPin *pReceivePin);
    virtual HRESULT Receive(IMediaSample *pSample, CTwoOneInputPin *ten);
    HRESULT InitializeOutputSample(__deref_out IMediaSample **ppOutSample);
    virtual HRESULT EndOfStream(void);
    virtual HRESULT BeginFlush(void);
    virtual HRESULT EndFlush(void);
    virtual HRESULT NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate);
protected:
	IMediaSample * pOutMediaSample;
	BOOL m_bEOSDelivered;
    BOOL m_bSampleSkipped;
    BOOL m_bQualityChanged;
    CCritSec m_csFilter;
    CCritSec m_csReceive;
    friend class CTwoOneInputPin;
    friend class CTwoOneOutputPin;
    CTwoOneInputPin *m_pInputL;
	CTwoOneInputPin *m_pInputR;
    CTwoOneOutputPin *m_pOutput;
};

#endif //_VTWOINP_H_
